@@ -5,17 +5,18 @@
 #include <config.h>
 #endif
 
-#ifdef USE_DEBUG
-#define DEBUG(format...) g_log(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, format)
-#define INFO_ON(format...) g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, format)
+#include <bits/wordsize.h>
+#if __WORDSIZE == 64
+typedef unsigned long int word_t;
 #else
-#define DEBUG(format...)
-#define INFO_ON(format...)
+typedef unsigned int word_t;
 #endif
 
-#if 0
-#define INFO(format...) printf(format);
+#ifdef USE_DEBUG
+#define DEBUG(format...) g_log(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, format)
+#define INFO(format...) g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, format)
 #else
+#define DEBUG(format...)
 #define INFO(format...)
 #endif
 
