@@ -1,15 +1,22 @@
-// site.h: interface for the CSite class.
-/////////////////////////////////////////////////////////////////////////////
-// Name:        site.h
-// Purpose:     Site Settings
-// Author:      PCMan (HZY)   http://pcman.ptt.cc/
-// E-mail:      pcman.tw@gmail.com
-// Created:     2004.07.15
-// Copyright:   (C) 2004 PCMan
-// Licence:     GPL : http://www.gnu.org/licenses/gpl.html
-// Modified by:
-/////////////////////////////////////////////////////////////////////////////
-
+/**
+ * site.h - interface for the CSite class.
+ *                                                                              
+ * Copyright (c) 2004-2005 PCMan <pcman.tw@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #if !defined(PCMANX_SITE_H)
 #define PCMANX_SITE_H
@@ -89,6 +96,15 @@ public:
 	bool m_bHorizontalCenterAlign;
 	bool m_bVerticalCenterAlign;
 
+#ifdef USE_PROXY
+	// Proxy settings
+	int    m_ProxyType;
+	string m_ProxyAddr;
+	int    m_ProxyPort;
+	string m_ProxyUser;
+	string m_ProxyPass;
+#endif
+
 	X_EXPORT CSite(string Name = "");
 	X_EXPORT ~CSite();
 
