@@ -33,7 +33,7 @@
 
 using namespace std;
 
-static gboolean on_key_pressed(GtkWidget* wnd, GdkEventKey *evt, CTermView* _this)
+static gboolean on_key_pressed(GtkWidget* wnd UNUSED, GdkEventKey *evt, CTermView* _this)
 {
 	bool ret = gtk_im_context_filter_keypress(_this->m_IMContext, evt );
 	if( !_this->PreKeyDown(evt) && !ret )
@@ -41,12 +41,12 @@ static gboolean on_key_pressed(GtkWidget* wnd, GdkEventKey *evt, CTermView* _thi
 	return ret;
 }
 
-static void on_im_commit(GtkIMContext *im, gchar *arg, CTermView* _this)
+static void on_im_commit(GtkIMContext *im UNUSED, gchar *arg, CTermView* _this)
 {
 	_this->OnTextInput(arg);
 }
 
-static gboolean on_mouse_down(GtkWidget* widget, GdkEventButton* evt, CTermView* _this)
+static gboolean on_mouse_down(GtkWidget* widget UNUSED, GdkEventButton* evt, CTermView* _this)
 {
 	switch(evt->button)
 	{
@@ -62,7 +62,7 @@ static gboolean on_mouse_down(GtkWidget* widget, GdkEventButton* evt, CTermView*
 	return true;
 }
 
-static gboolean on_mouse_up(GtkWidget* widget, GdkEventButton* evt, CTermView* _this)
+static gboolean on_mouse_up(GtkWidget* widget UNUSED, GdkEventButton* evt, CTermView* _this)
 {
 	switch(evt->button)
 	{
@@ -75,7 +75,7 @@ static gboolean on_mouse_up(GtkWidget* widget, GdkEventButton* evt, CTermView* _
 	return true;
 }
 
-static gboolean on_mouse_move(GtkWidget* widget, GdkEventMotion* evt, CTermView* _this)
+static gboolean on_mouse_move(GtkWidget* widget UNUSED, GdkEventMotion* evt, CTermView* _this)
 {
 	if (evt->is_hint)
 	{
@@ -89,13 +89,13 @@ static gboolean on_mouse_move(GtkWidget* widget, GdkEventMotion* evt, CTermView*
 	return true;
 }
 
-static gboolean on_mouse_scroll(GtkWidget* widget, GdkEventScroll* evt, CTermView* _this)
+static gboolean on_mouse_scroll(GtkWidget* widget UNUSED, GdkEventScroll* evt, CTermView* _this)
 {
 	_this->OnMouseScroll(evt);
 	return true;
 }
 
-void CTermView::OnBeforeDestroy( GtkWidget* widget, CTermView* _this)
+void CTermView::OnBeforeDestroy( GtkWidget* widget UNUSED, CTermView* _this)
 {
 	XftDrawDestroy( _this->m_XftDraw);
 	_this->m_XftDraw = NULL;
@@ -246,19 +246,19 @@ void CTermView::OnPaint(GdkEventExpose* evt)
 
 }
 
-void CTermView::OnSetFocus(GdkEventFocus* evt)
+void CTermView::OnSetFocus(GdkEventFocus* evt UNUSED)
 {
 	gtk_im_context_focus_in(m_IMContext);
 }
 
 
-bool CTermView::OnKeyDown(GdkEventKey* evt)
+bool CTermView::OnKeyDown(GdkEventKey* evt UNUSED)
 {
 	return true;
 }
 
 
-void CTermView::OnTextInput(const gchar* string)
+void CTermView::OnTextInput(const gchar* string UNUSED)
 {
     // Override this function to process text input.
 }
@@ -340,7 +340,7 @@ static void fixed_gdk_draw_trapezoids (GdkDrawable *drawable,
 }
 #endif
 
-bool CTermView::DrawSpaceFillingChar(const char* ch, int len, int x, int y, GdkRectangle* clip, GdkColor* clr)
+bool CTermView::DrawSpaceFillingChar(const char* ch, int len UNUSED, int x, int y, GdkRectangle* clip UNUSED, GdkColor* clr UNUSED)
 {
 	GdkDrawable* dc = m_Widget->window;
 	guchar* uchar = (guchar*)ch;
@@ -601,7 +601,7 @@ void CTermView::PointToLineCol(int *x, int *y, bool *left)
 	}
 }
 
-void CTermView::OnSize(GdkEventConfigure* evt)
+void CTermView::OnSize(GdkEventConfigure* evt UNUSED)
 {
 	if( !m_AutoFontSize || !m_pTermData )
 		return;
@@ -622,7 +622,16 @@ static int DrawCharWrapper( int row, int col, void *data )
 	return tv->DrawChar( row, col );
 }
 
-void CTermView::ExtendSelection( int row, int col, bool left )
+void CTermView::ClearSelection()
+{
+	m_CancelSel = true;
+
+	m_Caret.Hide();
+	m_pTermData->m_Sel->Unselect( DrawCharWrapper, this );
+	m_Caret.Show( false );
+}
+
+void CTermView::ExtendSelection( int row, int col, bool left UNUSED )
 {
 	row += m_pTermData->m_FirstLine;
 
@@ -704,17 +713,11 @@ void CTermView::OnLButtonDown(GdkEventButton* evt)
 	{
 		// clear the old selection
 		if( !m_pTermData->m_Sel->Empty() )
-		{
-			m_CancelSel = true;
-
-			m_Caret.Hide();
-			m_pTermData->m_Sel->Unselect( DrawCharWrapper, this );
-			m_Caret.Show( false );
-		}
+			ClearSelection();
 
 		SetCapture();
 
-		INFO("x=%d, y=%d, grab=%d\n", x, y, HasCapture());
+		INFO("x=%d, y=%d, grab=%d", x, y, HasCapture());
 
 		m_pTermData->m_Sel->NewStart( y, x, left,
 				(evt->state & (GDK_SHIFT_MASK|GDK_MOD1_MASK|GDK_CONTROL_MASK)) );
@@ -722,18 +725,18 @@ void CTermView::OnLButtonDown(GdkEventButton* evt)
 }
 
 
-void CTermView::OnMouseMove(GdkEventMotion* evt)
+void CTermView::OnMouseMove(GdkEventMotion* evt UNUSED)
 {
 
 }
 
 
-void CTermView::OnMouseScroll(GdkEventScroll* evt)
+void CTermView::OnMouseScroll(GdkEventScroll* evt UNUSED)
 {
 
 }
 
-void CTermView::OnRButtonDown(GdkEventButton* evt)
+void CTermView::OnRButtonDown(GdkEventButton* evt UNUSED)
 {
 
 }
@@ -768,18 +771,18 @@ void CTermView::OnLButtonUp(GdkEventButton* evt)
 
 }
 
-void CTermView::OnRButtonUp(GdkEventButton* evt)
+void CTermView::OnRButtonUp(GdkEventButton* evt UNUSED)
 {
 }
 
-void CTermView::OnKillFocus(GdkEventFocus *evt)
+void CTermView::OnKillFocus(GdkEventFocus *evt UNUSED)
 {
 	gtk_im_context_focus_out(m_IMContext);
 }
 
 // Preview the GdkEventKey before it's sent to OnKeyDown
 // regardless of the returned value of im_context_filter_key_press.
-bool CTermView::PreKeyDown(GdkEventKey *evt)
+bool CTermView::PreKeyDown(GdkEventKey *evt UNUSED)
 {
 	return false;
 }
@@ -808,13 +811,14 @@ void CTermView::OnBlinkTimer()
 	m_Caret.Blink();
 }
 
-void CTermView::OnMButtonDown(GdkEventButton* evt)
+void CTermView::OnMButtonDown(GdkEventButton* evt UNUSED)
 {
 	PasteFromClipboard(true);
 }
 
 
 string CTermView::m_s_ANSIColorStr;
+string CTermView::m_s_CharSet;
 
 void CTermView::PasteFromClipboard(bool primary)
 {
@@ -835,7 +839,7 @@ void CTermView::PasteFromClipboard(bool primary)
 		DoPasteFromClipboard(m_s_ANSIColorStr, true);
 }
 
-void CTermView::DoPasteFromClipboard(string text, bool contain_ansi_color)
+void CTermView::DoPasteFromClipboard(string text UNUSED, bool contain_ansi_color UNUSED)
 {
 
 }
@@ -846,26 +850,27 @@ void CTermView::CopyToClipboard(bool primary, bool with_color, bool trim)
 	if(!m_pTermData)
 		return;
 	m_s_ANSIColorStr = "";
-	if( with_color )
+	if( with_color ) {
 		text = m_pTermData->GetSelectedTextWithColor(trim);
-	else
+		m_s_ANSIColorStr = text;
+		m_s_CharSet = m_pTermData->m_Encoding;
+		INFO("copy(with color): %s", text.c_str());
+	}
+	else {
 		text = m_pTermData->GetSelectedText(trim);
+		gsize wl = 0;
+		const gchar* utext = g_convert_with_fallback(
+				text.c_str(), text.length(),
+				"utf-8", m_pTermData->m_Encoding.c_str(),
+				(gchar *) "?", NULL, &wl, NULL);
+		if(!utext)
+			return;
 
-	gsize wl = 0;
-	const gchar* utext = g_convert_with_fallback( text.c_str(), text.length(),
-			"utf-8", m_pTermData->m_Encoding.c_str(), "?", NULL, &wl, NULL);
-	if(!utext)
-		return;
-
-	if( with_color )
-		m_s_ANSIColorStr = string(utext);
-	else
-	{
 		GtkClipboard* clipboard = gtk_clipboard_get(  primary ? GDK_SELECTION_PRIMARY : GDK_NONE );
 		gtk_clipboard_set_text(clipboard, utext, wl );
+		INFO("copy(without color): %s", utext);
+		g_free((void*)utext);
 	}
-	INFO("select: %s", utext);
-	g_free((void*)utext);
 }
 
 void CTermView::GetCellSize( int &w, int &h )
@@ -1036,10 +1041,14 @@ bool CTermView::HyperLinkHitTest(int x, int y, int* start, int* end)
 	if( x > 0 && x < m_pTermData->m_ColsPerPage && pattr[x].IsHyperLink() )
 	{
 		int _start, _end;
-		for( _start = x-1; _start > 0 && pattr[_start].IsHyperLink(); _start-- );
+		for( _start = (x - 1); _start > 0 &&
+		                       pattr[_start].IsHyperLink(); _start-- )
+			;
 		if( !pattr[_start].IsHyperLink() )
 			_start++;
-		for( _end = x+1; _end < m_pTermData->m_ColsPerPage && pattr[_end].IsHyperLink(); _end++ );
+		for( _end = (x + 1); _end < m_pTermData->m_ColsPerPage &&
+		                     pattr[_end].IsHyperLink(); _end++ )
+			;
 		*start = _start;
 		*end = _end;
 		INFO("%d, %d : %d, %d", x, y, _start, _end);
@@ -1086,7 +1095,7 @@ void CTermView::RecalcCharDimension()
 	m_Caret.Show();
 }
 
-void CTermView::OnHyperlinkClicked(string url)	// Overriden in derived classes
+void CTermView::OnHyperlinkClicked(string url UNUSED)	// Overriden in derived classes
 {
 
 }
