@@ -1,10 +1,26 @@
+/**
+ * msgdata.cpp - Class of messages MAPs for Bot.
+ *
+ * Copyright (c) 2005 Youchen Lee ( utcr.org )
+ *                                                                              
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
-// Name:        msgdata.cpp
-// Purpose:     Class of messages MAPs for Bot.
-// Author:      Youchen Lee ( utcr.org ) 
-// Licence:     GPL : http://www.gnu.org/licenses/gpl.html
-
+#include <cstdlib>
 #include <unistd.h>
+#include <stdio.h>
 
 #include "nancy_bot/msgdata.h"
 #include "nancy_bot/botutil.h"
