@@ -1,14 +1,23 @@
-/////////////////////////////////////////////////////////////////////////////
-// Name:        telnetcon.h
-// Purpose:     Class dealing with telnet connections, parsing telnet commands.
-// Author:      PCMan (HZY)   http://pcman.ptt.cc/
-// E-mail:      pcman.tw@gmail.com
-// Created:     2004.7.16
-// Copyright:   (C) 2004 PCMan
-// Licence:     GPL : http://www.gnu.org/licenses/gpl.html
-// Modified by:
-/////////////////////////////////////////////////////////////////////////////
-
+/**
+ * telnetcon.h - Class dealing with telnet connections,
+ *               parsing telnet commands.
+ *
+ * Copyright (c) 2005 PCMan <pcman.tw@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef TELNETCON_H
 #define TELNETCON_H
@@ -233,7 +242,7 @@ private:
 class CDNSRequest
 {
 public:
-	CDNSRequest(CTelnetCon* con, string address, int port) 
+	CDNSRequest(CTelnetCon* con, string address, int port UNUSED) 
 		: m_pCon(con), m_Address(address), m_Running(false)
 	{
 	}
